;												
; fr ATmega8									
; Bernhard.Erfurt@gmx.de						
; 10/2023										
;												

;												
; LED ROT blinkt: Programm Check Error			
; LED ROT+GELB blinkt: Interrupt Error			
; LED ROT+GELB+GRN blinkt: WDR Error (nicht in dieser Version)	
;												

;												
; Anschlussbelegung PINS						
;												
; PB0 ==>										
; PB1 ==>										
; PB2 ==>										
; PB3 ==>										
; PB4 ==>										
; PB5 ==>										
; PB6 ==>										
; PB7 ==>										
;												
; PC0 ==> ADC0									
; PC1 ==> ADC1									
; PC2 ==>										
; PC3 ==>										
; PC4 ==> 										
; PC5 ==> BEEP									
;												
; PD0 ==> Display CLC							
; PD1 ==> Display DAT							
; PD2 ==> TASTE									
; PD3 ==> TASTE									
; PD4 ==> 										
; PD5 ==> LED GRN								
; PD6 ==> LED GELB								
; PD7 ==> LED ROT								
;												


; ATmega8 										
.include "m8def.inc"
.include "MACRO.asm"
; KONSTANTEN									
.equ TAKT							=16000000	; 16 MHz	
; ERROR CODES									
.equ ERROR_A					=01 
; ZEICHEN-DARSTELLUNG - 7-SEGMENT				
.equ ASCII_0					=0b00111111	; 0 
.equ ASCII_1					=0b00000110	; 1 
.equ ASCII_2					=0b01011011	; 2 
.equ ASCII_3					=0b01001111	; 3 
.equ ASCII_4					=0b01100110	; 4 
.equ ASCII_5					=0b01101101	; 5 
.equ ASCII_6					=0b01111101	; 6 
.equ ASCII_7					=0b00000111	; 7 
.equ ASCII_8					=0b01111111	; 8 
.equ ASCII_9					=0b01101111	; 9 

.equ ASCII_AUS					=0b00000000	; ''
.equ ASCII_PUNKT				=0b10000000	; . 
.equ ASCII_VOLL					=0b11111111	; 8.
.equ ASCII_MINUS				=0b01000000	; - 
.equ ASCII_GRAD					=0b01100011	;  

.equ ASCII_A					=0b01110111	; A 
.equ ASCII_B					=0b01111100	; B 
.equ ASCII_C					=0b01011000	; C 
.equ ASCII_D					=0b01011110	; D 
.equ ASCII_E					=0b01111001	; E 
.equ ASCII_F					=0b01110001	; F 
.equ ASCII_P					=0b01110011	; P
; BEEP											
.equ BEEP_PIN_NR					= 5
.equ BEEP_DDR						= DDRC
.equ BEEP_PORT						= PORTC
; LED-CP(Clock)									
.equ CLOCK_PIN_NR					= 0
.equ CLOCK_DDR						= DDRD
.equ CLOCK_PORT						= PORTD
; LED-DAT(Daten)								
.equ DAT_PIN_NR						= 1
.equ DAT_DDR						= DDRD
.equ DAT_PORT						= PORTD
; TASTE "A"										
.equ TASTE_A_PIN_NR					= 2
.equ TASTE_A_PIN					= PIND
.equ TASTE_A_DDR					= DDRD
.equ TASTE_A_PORT					= PORTD
; TASTE "B"										
.equ TASTE_B_PIN_NR					= 3
.equ TASTE_B_PIN					= PIND
.equ TASTE_B_DDR					= DDRD
.equ TASTE_B_PORT					= PORTD


; LED-GRUEN										
.equ GRUEN_PIN_NR					= 5
.equ GRUEN_DDR						= DDRD
.equ GRUEN_PORT						= PORTD
; LED-GELB										
.equ GELB_PIN_NR					= 6
.equ GELB_DDR						= DDRD
.equ GELB_PORT						= PORTD
; LED-ROT										
.equ ROT_PIN_NR						= 7
.equ ROT_DDR						= DDRD
.equ ROT_PORT						= PORTD


; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						=SRAM_START		; 0x0060 START Adresse 		

.equ adr_DIGIT_NR					=SRAM_START+5	; 0...7						

.equ adr_DIGIT0						=SRAM_START+10	; unten rechts				
.equ adr_DIGIT1						=SRAM_START+11	
.equ adr_DIGIT2						=SRAM_START+12	
.equ adr_DIGIT3						=SRAM_START+13	
.equ adr_DIGIT4						=SRAM_START+14	
.equ adr_DIGIT5						=SRAM_START+15	
.equ adr_DIGIT6						=SRAM_START+16	
.equ adr_DIGIT7						=SRAM_START+17	

.equ adr_MENUE						=SRAM_START+100
.equ adr_TASTE						=SRAM_START+101
.equ adr_ERROR_CODE					=SRAM_START+102		; ERROR CODE vom ersten auftretenden Fehler		

.equ adr_U							=SRAM_START+110
.equ adr_i							=SRAM_START+111
.equ adr_P_L						=SRAM_START+112
.equ adr_P_H						=SRAM_START+113
.equ adr_P_MAX_L					=SRAM_START+114
.equ adr_P_MAX_H					=SRAM_START+115

.equ adr_ADC_U_L					=SRAM_START+150
.equ adr_ADC_U_H					=SRAM_START+151
.equ adr_ADC_i_L					=SRAM_START+152
.equ adr_ADC_i_H					=SRAM_START+153


.equ adr_SRAM_CHECK					=SRAM_START+943	; SRAM CHECK muss immer "S" sein	

; REGISTER										
.def ERROR		= R8
.def TIMER0		= R9

.def NULL 		= R12
.def EINS 		= R14
.def VOLL		= R15

.def temp 		= R16
.def temp1 		= R17
.def temp2 		= R18
.def temp3 		= R19
.def temp4 		= R20
.def temp5 		= R21
.def temp6 		= R22
.def temp7 		= R23
.def temp8 		= R24
.def temp9 		= R25


.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   ; Timer1 Overflow Handler           	
rjmp TIM0_OVF   		; Timer0 Overflow Handler           					
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	
;-------------------------------------------------------------------------------
; Unkorrekten Interrupt Aufruf 													
INTERRUPT_ERROR:
	GELB_ON
	ROT_ON
	rcall wait_100ms
	GRUEN_OFF
	GELB_OFF
	ROT_OFF
	rcall wait_100ms
	rjmp INTERRUPT_ERROR
;-------------------------------------------------------------------------------
RESET:                        
	ldi temp, LOW(RAMEND)					; STACK initialisieren								
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
	rcall WDR_INITIALISIERUNG				; WDR_INITIALISIERUNG								
	rcall PORT_INITIALISIERUNG				; PORT_CLEAR										
	rcall SRAM_CLEAR						; SRAM lschen										
	rcall STARTWERTE						; Startwerte										
	rcall ADC_INITIALISIERUNG				; ADC_INITIALISIERUNG								
	rcall TIMER0_INITIALISIERUNG_1ms		; TIMER												
	rcall TIMER_AKTIVIEREN					; TIMER												
	rcall EEPROM_LOAD
	rcall PROGRAMM_CHECK					; CHECK												
	sei										; Interrupts freigeben								
;-------------------------------------------------------------------------------
	rcall BEGRUESSUNG
	rcall WAIT_1s
	rcall BEEP_KURZ
	rcall WAIT_100ms
	rcall BEEP_KURZ
	rcall WAIT_100ms
	rcall BEEP_KURZ
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR										; Watch-Dog-RESET	
	rcall PROGRAMM_CHECK
	rcall ADC_U
	rcall WAIT_100ms
	rcall ADC_I
	rcall BERECHNUNG_P
	rcall BERECHNUNG_P_MAX
	rcall MENUE
	rcall TASTEN_WAIT
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
BERECHNUNG_P:
	LDS temp1,(adr_U)
	LDS temp2,(adr_i)
	mul temp1,temp2
	mov temp1,R0
	mov temp2,R1
	rcall DIVISION_16_100 ; durch 100
	STS(adr_P_L),temp1
	STS(adr_P_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
BERECHNUNG_P_MAX:
	LDS temp1,(adr_P_L)
	LDS temp2,(adr_P_H)
	LDS temp3,(adr_P_MAX_L)
	LDS temp4,(adr_P_MAX_H)
;-------------------------------------------------------------------------------
	cp temp2,temp4
	breq BERECHNUNG_P_L
	brsh BERECHNUNG_P_GROESSER
	brlo BERECHNUNG_P_NIX
;-------------------------------------------------------------------------------
BERECHNUNG_P_L:
	cp temp1,temp3
	breq BERECHNUNG_P_NIX
	brsh BERECHNUNG_P_GROESSER
	brlo BERECHNUNG_P_NIX
;-------------------------------------------------------------------------------
BERECHNUNG_P_NIX:
	ret
;-------------------------------------------------------------------------------
BERECHNUNG_P_GROESSER:
	STS(adr_P_MAX_L),temp1
	STS(adr_P_MAX_H),temp2
	rcall EEPROM_SAVE
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "ADC.asm"
.include "TIMER.asm"
.include "LED.asm"
.include "EEPROM.asm"
.include "SONSTIGES.asm"
.include "MENUE.asm"
.include "WAIT.asm"
.include "ZAHLEN.asm"

